/*
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:    lo2.cs

   Purpose: Demonstrates how to solve small linear
            optimization problem using the MOSEK C# API.
 */


using System;

public class lo2
{  
    public static void Main ()
    {              
        // Since the value infinity is never used, we define
        // 'infinity' symbolic purposes only
        double
          infinity = 0;
        
        const int NUMCON = 3;
        const int NUMVAR = 4;
        const int NUMANZ = 9;
        
        double[] c    = {3.0, 1.0, 5.0, 1.0};

        mosek.boundkey[] bkc  = {mosek.boundkey.fx,
                                 mosek.boundkey.lo,
                                 mosek.boundkey.up};
        double[] blc  = {30.0,
                          15.0,
                          -infinity};
        double[] buc  = {30.0,
                          +infinity,
                          25.0};
        
        mosek.boundkey[]  bkx  = {mosek.boundkey.lo,
                          mosek.boundkey.ra,
                          mosek.boundkey.lo,
                          mosek.boundkey.lo};
        double[]  blx  = {0.0,
                          0.0,
                          0.0,
                          0.0};
        double[]  bux  = {+infinity,
                          10.0,
                          +infinity,
                          +infinity};

        int[][]
          asub = new int[NUMVAR][];
        
        asub[0] = new int[] {0, 1};
        asub[1] = new int[] {0, 1, 2};
        asub[2] = new int[] {0, 1};
        asub[3] = new int[] {1, 2};        

        double[][]
          aval   = new double[NUMVAR][];

        aval[0] = new double[] { 3.0, 2.0 };
        aval[1] = new double[] { 1.0, 1.0, 2.0};
        aval[2] = new double[] { 2.0, 3.0};
        aval[3] = new double[] { 1.0, 3.0};

        double[] xx  = new double[NUMVAR];
        
        mosek.Task 
            task = null;
        mosek.Env  
            env  = null;
        
        try
          {
            // Make mosek environment. 
            env  = new mosek.Env ();
            // Initialize the environment.
            env.init ();
            // Create a task object linked with the environment env.
            task = new mosek.Task (env, NUMCON,NUMVAR);
        

             /* Give MOSEK an estimate on the size of
                   the data to input. This is done to increase
                   the speed of inputing data and is optional.*/
                
            task.putmaxnumvar(NUMVAR); 
            task.putmaxnumcon(NUMCON);      
            task.putmaxnumanz(NUMANZ);

            /* Append the constraints. */
            task.append(mosek.accmode.con,NUMCON);

            /* Append the variables. */
            task.append(mosek.accmode.var,NUMVAR);

            /* Put C. */
            task.putcfix(0.0);
            for(int j=0; j<NUMVAR; ++j)
              task.putcj(j,c[j]);

            /* Put constraint bounds. */
            for(int i=0; i<NUMCON; ++i)
              task.putbound(mosek.accmode.con,i,bkc[i],blc[i],buc[i]);

            /* Put variable bounds. */
            for(int j=0; j<NUMVAR; ++j)
              task.putbound(mosek.accmode.var,j,bkx[j],blx[j],bux[j]);

            /* Put A. */
            if ( NUMCON>0 )
            {
              for(int j=0; j<NUMVAR; ++j)
                task.putavec(mosek.accmode.var,
                             j,
                             asub[j],
                             aval[j]);
            }

            task.putobjsense(mosek.objsense.maximize);

            try
              {       
                task.optimize();
              }
            catch (mosek.Warning w)
              {
                Console.WriteLine("Mosek warning:");
                Console.WriteLine (w.Code);
                Console.WriteLine (w);
              }
            
            task.getsolutionslice(mosek.soltype.bas, /* Basic solution.       */
                                  mosek.solitem.xx,  /* Which part of solution.  */
                                  0,                 /* Index of first variable. */
                                  NUMVAR,            /* Index of last variable+1 */
                                  xx);

            for(int j = 0; j < NUMVAR; ++j)
              Console.WriteLine ("x[{0}]:{1}", j,xx[j]);
          }
        catch (mosek.Exception e)
            {
                Console.WriteLine (e.Code);
                Console.WriteLine (e);
            }

        if (task != null) task.Dispose ();
        if (env  != null)  env.Dispose ();
    }
}
