/* 
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:      qcqo1.cs

   Purpose:   Demonstrate how to solve a quadratic
              optimization problem using the MOSEK API.

              minimize  x0^2 + 0.1 x1^2 +  x2^2 - x0 x2 - x1 
              s.t 1 <=  x0 + x1 + x2 - x0^2 - x1^2 - 0.1 x2^2 + 0.2 x0 x2 
              x >= 0                                     
*/


using System;

public class qcqo1
{
  public static void Main ()
  {       
    const double inf = 0.0; 

    const int NUMCON = 1;   /* Number of constraints.             */
    const int NUMVAR = 3;   /* Number of variables.               */
    const int NUMANZ = 3;   /* Number of numzeros in A.           */
    const int NUMQNZ = 4;   /* Number of nonzeros in Q.           */
    
    mosek.boundkey[] 
      bkc = { mosek.boundkey.lo },
      bkx = { mosek.boundkey.lo, mosek.boundkey.lo, mosek.boundkey.lo };
    int[] 
        ptrb = { 0, 1, 2 },
        ptre = { 1, 2, 3 },
        sub  = { 0, 0, 0 };
    double[]    
      blc  = { 1.0 },
      buc  = { inf },
      c    = { 0.0, -1.0, 0.0 },
      blx  = { 0.0, 0.0, 0.0 },
      bux  = { inf, inf, inf },
      val  = { 1.0, 1.0, 1.0 },
      xx   = new double[NUMVAR];
    mosek.Task 
      task = null;
    mosek.Env
      env = null;

    try
    {
      env  = new mosek.Env ();
      env.init ();
      task = new mosek.Task (env, NUMCON,NUMVAR);
      /* Define bounds for the constraint. */

      /* Constraint: 0 */
      bkc[0] = mosek.boundkey.lo;
      blc[0] = 1.0;
      buc[0] = mosek.Val.infinity;

      /* Define information for the variables. */
            
      /* Variable: x0 */
      c[0]    = 0.0;
            
      /* Constraint matrix. */
      ptrb[0] = 0;  ptre[0] = 1;
      sub[0]  = 0;  val[0]  = 1.0;

      /* Bounds. */
      bkx[0]  = mosek.boundkey.lo;
      blx[0]  = 0.0;
      bux[0]  = mosek.Val.infinity;

      /* Variable: x1 */
      c[1]    = -1.0;

      ptrb[1] = 1;  ptre[1] = 2;
      sub[1]  = 0;  val[1]  = 1.0;

      bkx[1]  = mosek.boundkey.lo;
      blx[1]  = 0.0;
      bux[1]  = mosek.Val.infinity;

      /* Variable: x2 */
      c[2]    = 0.0;

      ptrb[2] = 2;  ptre[2] = 3;
      sub[2]  = 0;  val[2]  = 1.0;
            
      bkx[2]  = mosek.boundkey.lo;
      blx[2]  = 0.0;
      bux[2]  = mosek.Val.infinity;
 
      task.inputdata(NUMCON,NUMVAR,
                     c,0.0,
                     ptrb,
                     ptre,
                     sub,
                     val,
                     bkc,
                     blc,
                     buc,
                     bkx,
                     blx,
                     bux);
      /*
       * The lower triangular part of the Q
       * matrix in the objective is specified.
       */
            
      {
        int[]
          qsubi = { 0, 1, 2, 2 },
          qsubj = { 0, 1, 0, 2 };
        double[]
          qval = { 2.0, 0.2, -1.0, 2.0 };

        /* Input the Q for the objective. */

        task.putqobj(qsubi,qsubj,qval);
      }      
      /*
       * The lower triangular part of the Q^0
       * matrix in the first constraint is specified.
       * This corresponds to adding the term
       *  - x0^2 - x1^2 - 0.1 x2^2 + 0.2 x0 x2
       */
      {
        int[]         
          qsubi = { 0, 1, 2, 2 },
          qsubj = { 0, 1, 2, 0 };
        double[]
          qval = { -2.0, -2.0, -0.2, 0.2 };
        
        /* put Q^0 in constraint with index 0. */
      
        task.putqconk (0,
                       qsubi, 
                       qsubj, 
                       qval); 
      }

      task.putobjsense(mosek.objsense.minimize);

      task.optimize();

      task.getsolutionslice(mosek.soltype.itr,
                            mosek.solitem.xx,
                            0,
                            NUMVAR,
                            xx);
	    
      Console.WriteLine ("Primal solution");
      for ( int j=0; j<NUMVAR; ++j )
        Console.Write("x[{0}]: {1}\n",j,xx[j]);
    }
    catch (mosek.Exception e)
    {
      Console.WriteLine (e);
    }
    if (task != null) task.Dispose ();
    if (env  != null)  env.Dispose ();

  } /* Main */
}
