##
#    Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.
#
#    File:      qcqo1.py
#
#    Purpose:   Demonstrate how to solve a quadratic
#               optimization problem using the MOSEK API.
#
#               minimize  x0^2 + 0.1 x1^2 +  x2^2 - x0 x2 - x1 
#               s.t 1 <=  x0 + x1 + x2 - x0^2 - x1^2 - 0.1 x2^2 + 0.2 x0 x2 
#               x >= 0
##


import sys
import os

import pymosek as mosek

from mosekarr import array, Float,zeros,ones

# Since the actual value of Infinity is ignores, we define it solely
# for symbolic purposes:
inf = 0.0

# Define a stream printer to grab output from MOSEK
def streamprinter(text):
    sys.stdout.write(text)
    sys.stdout.flush()

# We might write everything directly as a script, but it looks nicer
# to create a function.
def main ():
    # Open MOSEK and create an environment and task
    # Create a handle to MOSEK
    mskhandle = mosek.mosek ()
    # Make a MOSEK environment
    env = mskhandle.Env ()
    # Attach a printer to the environment
    env.set_Stream (mosek.streamtype.log, streamprinter)
    # Initialize the environment
    env.init ()
    # Create a task
    task = env.Task()

    # Set up and input bounds and linear coefficients
    bkc   = array([ mosek.boundkey.lo ])
    blc   = array([ 1.0 ])
    buc   = array([ inf ])
    
    bkx   = array([ mosek.boundkey.lo,
                    mosek.boundkey.lo,
                    mosek.boundkey.lo ])
    blx   = array([ 0.0,  0.0, 0.0 ])
    bux   = array([ inf,  inf, inf ])

    c     = array([ 0.0, -1.0, 0.0 ])

    asub  = array([   0,   0,   0 ])
    aval  = array([ 1.0, 1.0, 1.0 ])
    aptrb = array([ 0, 1, 2 ])
    aptre = array([ 1, 2, 3 ])

    numvar = len(bkx)
    numcon = len(bkc)

##REF:func:task:inputdata
    task.inputdata(numcon,numvar,
                   c,0.0,
                   aptrb,
                   aptre,
                   asub,
                   aval,
                   bkc,
                   blc,
                   buc,
                   bkx,
                   blx,
                   bux)

    # Set up and input quadratic objective

    qsubi = array([ 0,   1,    2,   2   ])
    qsubj = array([ 0,   1,    0,   2   ])
    qval  = array([ 2.0, 0.2, -1.0, 2.0 ])

    task.putqobj(qsubi,qsubj,qval)
      
    # The lower triangular part of the Q^0
    # matrix in the first constraint is specified.
    # This corresponds to adding the term
    # - x0^2 - x1^2 - 0.1 x2^2 + 0.2 x0 x2
                
    qsubi = array([  0,    1,    2,   2   ])
    qsubj = array([  0,    1,    2,   0   ])
    qval  = array([ -2.0, -2.0, -0.2, 0.2 ])

    # put Q^0 in constraint with index 0. 
      
##REF:func:task:putqconk
    task.putqconk (0, qsubi,qsubj, qval); 

    task.putobjsense(mosek.objsense.minimize)

    # Optimize
    task.optimize()


    # Fetch the interior solution
    xx = zeros(numvar,Float)
##REF:enum:soltype:itr
##REF:enum:solitem:xx
    task.getsolutionslice(mosek.soltype.itr,
                          mosek.solitem.xx,
                          0,numvar,
                          xx)
	    
    print "Primal interior solution: "
    print "\t",xx

    return None


# call the main function
try:
    main ()
except mosek.Exception, err:
    print "ERROR: %s" % str(err)
    sys.exit(1)
except:
    import traceback
    traceback.print_exc()
    sys.exit(1)
